#!/bin/sh
# (c) 2004 Team Mirage by [BiM]gtk <gtk@gtk.no>
# startserver.sh - for running a seals server in Linux.


# ---------------------------------------------------------------------------

# Edit these configuration options to fit your system and seals installation.

# You should make a seperate user account on your Linux system specifically 
# for running the seals server. Never run gameservers as root!

# The full path to where Quake III Arena is installed.
QUAKE3PATH="/home/quake3/quake3"

# The server configuration file, it _HAS_ to be inside the "seals" directory.
SERVERCONFIG="server.cfg"

# IP address the server listens on. Simply use "" to listen on all interfaces. 
NET_IP=""

# UDP network port to bind the server to.
NET_PORT="27960"

# When running multiple servers from the same q3a+seals installation it is
# vital that you make sure to have a seperate homepath for each server or you 
# will mix up the punkbuster files, punkbuster logfiles as well as the regular
# serverlogs and cause a mess. The reason is that problems will arise when two 
# or more seals servers decide to try to update the punkbuster software 
# simultaneously. In addition a mixed log of several servers will wreak havoc 
# for any type of log or stats parser trying to make sense of logdata. 
# Finally you should always specify the full path to the homepath directory!
HOMEPATH="/home/quake3/.q3a"

# Full path to qstat, http://www.qstat.org <- if your vendor doesnt package it.
QSTAT="/usr/local/bin/qstat"
# The binary is called quakestat on Debian Linux
#QSTAT="/usr/bin/quakestat"

# Map to load initially, should be the first map in your configs/mapcycle.cfg
MAP="ns_sleepingwolf"

# No need to edit anything below here.

# ---------------------------------------------------------------------------


function start()
{
    if [ -r "$PFILE" ]; then
        RUNNING=""
        /bin/ps -p `cat $PFILE` &>/dev/null && RUNNING="yes"
        
        if [ -n "$RUNNING" ]; then
            echo "seals already running as pid `cat $PFILE`"
            exit
        else
            echo "found pidfile but seals not running, deleting stale pidfile"
            rm -f $PFILE
        fi
    fi

    $QUAKE3PATH/q3ded \
    +set cvar_restart \
    +set fs_basepath $QUAKE3PATH \
    +set fs_game seals \
    +set vm_game 2 \
    +set sv_pure 1 \
    +set g_gametype 3 \
    +set dedicated 2 \
    +set ttycon 0 \
    +set net_ip $NET_IP \
    +set net_port $NET_PORT \
    +set fs_homepath $HOMEPATH \
    +exec $SERVERCONFIG \
    +map $MAP &>/dev/null &
    
    SEALSPID=$!

    echo "seals started with map $MAP and pid $SEALSPID"
    echo $SEALSPID >$PFILE
}

function have_pidfile()
{
    if [ -f "$PFILE" ]; then
        # found pidfile
        true
    else 
        echo "aborting, no seals running or missing pidfile"
        exit
    fi
}

function stop()
{
    have_pidfile
    echo "stopping seals"
    kill "`cat $PFILE`"
    rm -f $PFILE 
}

function usage()
{
    echo "usage $0: [options]" 
    echo "start       - starts seals server in the background"
    echo "stop        - stops seals server"
    echo "restart     - restarts seals server on the same map it was running"
    echo "status      - show the current server status"
}

function have_qstat()
{
    if [ -x "$QSTAT" ]; then
        # found qstat 
        true 
    else
        echo "aborting, qstat not found"
        exit
    fi
}

function restart()
{
    have_qstat
    have_pidfile
    
    echo "querying $NET_IP:$NET_PORT with $QSTAT"
    RAWQSTAT="`$QSTAT -nh -q3s $NET_IP:$NET_PORT`"
    PLAYERS="`echo $RAWQSTAT | awk '{ print $2 }' | cut -f1 -d "/"`"
    MAP="`echo $RAWQSTAT | sed -e 's/\/ //' | awk '{ print $3 }'`"

    echo "server currently has $PLAYERS players on $MAP"
       
    if [ "$PLAYERS" == "0" -o "$PLAYERS" == "ERROR" -o "$PLAYERS" == "DOWN" ]; then
        echo "attempting restart"
        stop
        start
    else
        echo "server not empty, not restarting"
    fi 
}

function status()
{
    $QSTAT -P -q3s $NET_IP:$NET_PORT
}

# Main script starts here

# path to seals (leave as default)
SEALSPATH="$QUAKE3PATH/seals"

# the path of the pidfile, must be writable (leave as default)
PIDFILEPATH="$SEALSPATH"

PFILE="$PIDFILEPATH/seals-"`echo $SERVERCONFIG | cut -f1 -d"."`"-$NET_PORT.pid"

if [ -z "$NET_IP" ]; then
    # assume localhost if emtpy
    NET_IP="localhost"
fi

# parse options

case $1 in
    start) start ;;
    stop) stop ;;
    restart) restart ;;
    status) status ;;
    *) usage
esac

# eof
